@@ -6,20 +6,14 @@ use curry;
 use MIDI::RtController ();
 use MIDI::RtController::Filter::CC ();
 
-my $input_name  = shift || 'joystick';
-my $output_name = shift || 'usb';
-
-my $control = MIDI::RtController->new(
-    input   => $input_name,
-    output  => $output_name,
-    verbose => 1,
-);
+my $input_names = shift || 'keyboard,pad,joystick'; # midi controller devices
+my $output_name = shift || 'usb'; # midi output
 
 my %filters = (
-    # 1 => { # mod-wheel
-        # type => 'breathe',
+    1 => { # mod-wheel
+        type => 'breathe',
         # time_step => 0.1,
-    # },
+    },
     # 13 => { # delay time
         # type => 'breathe',
         # time_step => 0.5,
@@ -50,11 +44,30 @@ my %filters = (
     # },
 );
 
-for my $ctrl (keys %filters) {
-    my %params = $filters{$ctrl}->%*;
+my $inputs = [ split /,/, $input_names ];
+
+my $control = MIDI::RtController->new(
+    input   => $inputs->[0],
+    output  => $output_name,
+    verbose => 1,
+);
+
+# open the remaining inputs
+for my $name (@$inputs[1 .. $#$inputs]) {
+    MIDI::RtController->new(
+        input    => $name,
+        loop     => $control->loop,
+        midi_out => $control->midi_out,
+        verbose  => 1,
+    );
+}
+
+# add the filters
+for my $cc (keys %filters) {
+    my %params = $filters{$cc}->%*;
     my $type = delete $params{type};
     my $filter = MIDI::RtController::Filter::CC->new(rtc => $control);
-    $filter->control($ctrl);
+    $filter->control($cc);
     for my $param (keys %params) {
         $filter->$param($params{$param});
     }
